/***************************************************************************
 *   Copyright (C) 2010 by Stefan Baur                                     *
 *   kwatersaver@c0n.de                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA            *
 ***************************************************************************/

#include <kaboutapplicationdialog.h>
#include "kwatersaver4.h"
#include <QDesktopWidget>
#include "settings.h"

#include <QDebug>

static KAboutData* s_aboutData = 0;

class KWaterSaver4Interface : public KScreenSaverInterface
{
public:
	virtual KAboutData* aboutData() {
		return s_aboutData;
	}

	/** function to create screen saver object */
	virtual KScreenSaver* create( WId id )
	{
		return new KWaterSaver4( id );
	}

	/** function to create setup dialog for screen saver */
	virtual QDialog* setup()
	{
		return new KWaterSaver4Setup();
	}
};


int main( int argc, char*argv[] )
{
	s_aboutData = new KAboutData( "kwatersaver4.kss", 0,
								ki18n("KWaterSaver4"), "0.7",
								ki18n("An OpenGL screensaver that puts your desktop under water."),
								KAboutData::License_GPL,
								ki18n("Copyright (C) 2010 Stefan Baur"),
								KLocalizedString(),
								"http://kwatersaver.c0n.de/",
								"kwatersaver@c0n.de" );
	s_aboutData->addAuthor( ki18n("Stefan Baur"),
							ki18n("Original author of <a href=\"http://kwatersaver.c0n.de\">KWaterSaver</a>."),
							"kwatersaver@c0n.de");

	KWaterSaver4Interface kss;
	return kScreenSaverMain( argc, argv, kss );
}


//-----------------------------------------------------------------------------
//! dialog to setup screen saver parameters
KWaterSaver4Setup::KWaterSaver4Setup( QWidget *parent ) : KDialog( parent )
{
	setCaption( i18n( "Setup KWaterSaver Screen Saver" ) );
	setButtons( Ok|Cancel|Help|Default );
	setDefaultButton( Ok );
	setModal( true );
	showButtonSeparator(true);
	setButtonText( Help, i18n( "A&bout" ) );

	QWidget *main = new QWidget( this );
	setMainWidget( main );
	ui.setupUi( main );
	saver = new KGLWaterSaver4( ui.preview );
	saver->resize( ui.preview->size() );
	saver->start();

	initDialog();  // Do this after we have a saver

	connect( this, SIGNAL( okClicked() ), SLOT( slotOkPressed() ) );
	connect( this, SIGNAL( helpClicked() ), SLOT( slotHelp() ) );
	connect( this, SIGNAL( defaultClicked() ), SLOT( slotDefault() ) );
	
	connect( ui.spinFPS, SIGNAL(valueChanged(int)), this, SLOT( updateFPS(int)) );
	
	connect( ui.spinWaterResX, SIGNAL(valueChanged(int)), this, SLOT(updateWaterResX(int)) );
	connect( ui.spinWaterResY, SIGNAL(valueChanged(int)), this, SLOT(updateWaterResY(int)) );
	
	connect( ui.checkRenderLighting, SIGNAL(toggled(bool)), this, SLOT(toggleRenderLighting(bool)) );
	connect( ui.checkRenderEnvironment, SIGNAL(toggled(bool)), this, SLOT(toggleRenderEnvironment(bool)) );
	
	connect( ui.checkEffectRain, SIGNAL(toggled(bool)), this, SLOT(toggleEffectRain(bool)) );
	connect( ui.checkEffectWhirl, SIGNAL(toggled(bool)), this, SLOT(toggleEffectWhirl(bool)) );
	connect( ui.checkEffectBlob, SIGNAL(toggled(bool)), this, SLOT(toggleEffectBlob(bool)) );
	
	connect( ui.urlBackgroundFile, SIGNAL(textChanged(QString)), this, SLOT(updateBackgroundSource(QString)) );
	connect( ui.radioBackgroundCapture, SIGNAL(toggled(bool)), this, SLOT(setBackgroundSourceDesktop(bool)) );
	connect( ui.radioBackgroundFile, SIGNAL(toggled(bool)), this, SLOT(setBackgroundSourceFile(bool)) );
}

KWaterSaver4Setup::~KWaterSaver4Setup()
{
	delete saver;
}

void KWaterSaver4Setup::slotDefault()
{
	settings.defaults();
	
		// block signals before we update widgets
	ui.spinFPS->blockSignals(true);

	initDialog();

		// unblock
	ui.spinFPS->blockSignals(false);

		// saver now needs to process a few things
	if(saver)
	{
		saver->stop();
		saver->start();
	}
}

// The preview will have already read things into settings
void KWaterSaver4Setup::initDialog()
{
	qDebug()<<"KWaterSaver4Setup::initDialog()";
	
	ui.spinFPS->setValue(settings.fps);
	
	ui.spinWaterResX->setValue(settings.waterResX);
	ui.spinWaterResY->setValue(settings.waterResY);
	
	if(settings.bgSource=="desktop")
	{
		ui.radioBackgroundCapture->toggle();
	}
	else
	{
		ui.urlBackgroundFile->setText(settings.bgSource);
		ui.radioBackgroundFile->toggle();
	}
	
	ui.checkRenderLighting->setChecked(settings.renderLighting);
	ui.checkRenderEnvironment->setChecked(settings.renderEnvironment);

	ui.checkEffectRain->setChecked(settings.effectRain);
	ui.checkEffectWhirl->setChecked(settings.effectWhirl);
	ui.checkEffectBlob->setChecked(settings.effectBlob);
}


// Ok pressed - save settings and exit
void KWaterSaver4Setup::slotOkPressed()
{
	settings.save();
	accept();
}

void KWaterSaver4Setup::slotHelp()
{
	KAboutApplicationDialog mAbout(s_aboutData, this);
	mAbout.exec();
}

void KWaterSaver4Setup::updateFPS( int fps )
{
	settings.fps=fps;

	if(saver)
	{
		saver->stop();
		saver->start();
	}
}

void KWaterSaver4Setup::updateWaterResX(int x)
{
	settings.waterResX=x;

	if(saver)
		saver->reload();
}

void KWaterSaver4Setup::updateWaterResY(int y)
{
	settings.waterResY=y;
	
	if(saver)
		saver->reload();
}


void KWaterSaver4Setup::updateBackgroundSource(QString src)
{
	settings.bgSource=src;
	
	if(saver)
		saver->reload();
}

void KWaterSaver4Setup::setBackgroundSourceFile(bool on)
{
	if(on)updateBackgroundSource(ui.urlBackgroundFile->text());
}

void KWaterSaver4Setup::setBackgroundSourceDesktop(bool on)
{
	if(on)
	{
		settings.bgSource="desktop";
	
		if(saver)
			saver->reload();
	}
}

void KWaterSaver4Setup::toggleEffectRain(bool on)
{
	settings.effectRain=on;
}

void KWaterSaver4Setup::toggleEffectWhirl(bool on)
{
	settings.effectWhirl=on;
}

void KWaterSaver4Setup::toggleEffectBlob(bool on)
{
	settings.effectBlob=on;
}

void KWaterSaver4Setup::toggleRenderLighting(bool on)
{
	settings.renderLighting=on;
//	if(saver)
//		saver->reload();
}

void KWaterSaver4Setup::toggleRenderEnvironment(bool on)
{
	settings.renderEnvironment=on;
//	if(saver)
//		saver->reload();
}




//-----------------------------------------------------------------------------


KWaterSaver4::KWaterSaver4( WId id ) : KScreenSaver( id )
{
	setAttribute( Qt::WA_NoSystemBackground );

	kglwatersaver4 = new KGLWaterSaver4();

	embed(kglwatersaver4);

	kglwatersaver4->start();

	// Show last to avoid occasional flicker on startup
	kglwatersaver4->show();
}

void KWaterSaver4::resizeEvent( QResizeEvent * )
{
	if ( kglwatersaver4 )
	{
		kglwatersaver4->stop();
		kglwatersaver4->resize( width(), height() );
		kglwatersaver4->start();
	}
}

KWaterSaver4::~KWaterSaver4()
{
	delete kglwatersaver4;
}

